#![no_std]
#![no_main]
#![feature(abi_x86_interrupt)]
#![feature(associated_type_defaults)]

use events::EVENTMAN;

use crate::events::Event;

#[macro_use]
mod logger;
mod interrupts;
mod gdt;
mod utils;
#[macro_use]
mod events;
mod mem;




#[no_mangle]
extern "C" fn kmain(mb2_info_addr: usize) -> ! {
    logger::init(log::LevelFilter::Trace).unwrap();
    gdt::init();
    interrupts::init_idt();
    let mem_info = mem::MemInfo::load(mb2_info_addr);
    let mut _alloc = mem::area_frame_alloc::AreaFrameAllocator::new(mem_info);
    
    EVENTMAN.lock().add_listener(events::Event::Ps2KeyPress(None), |k| {
        let Event::Ps2KeyPress(v) = k else {panic!()};

        log::debug!("Keypress event received! ({:?})", v.unwrap());
        Ok(())
    });


    // let (level_4_page_table, _) = Cr3::read();
    // println!("Level 4 page table at: {:?}", level_4_page_table.start_address());


    utils::hcf();
}


#[panic_handler]
fn panic(pi: &core::panic::PanicInfo) -> ! {
    log::error!("{pi}");
    utils::hcf();
}


